package inet.warehouse.web;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletException;
import java.io.IOException;
import java.util.Hashtable;

public class ActionServlet extends HttpServlet {
    /**
     * Executes the appropriate action
     * @param request
     * @param response
     * @throws ServletException when unable to execute the specified action
     * @throws IOException ???
     */
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        try {
            initializeInitParams();

            Action action;
            String uri = request.getRequestURI();

            action = getAction(uri);

            String result = action.executeAction(request, response);
            //result = WebUtils.fixContextRelativeUrl(request, result);
            if ((result != null) && (result.trim().length() > 0)) {
                request.getRequestDispatcher(result).forward(request, response);
            } else {
                throw new ServletException("Action " + uri + " returned empty result.");
            }
        } catch (Exception e) {
            throw new ServletException(e);
        }
    }


    /**
     * Dummy implementation, calls doPost
     * @param request
     * @param response
     * @throws ServletException
     * @throws IOException
     */
    protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        doPost(request, response);
    }

    /**
     * Gets the action name from the requested URI, checks if the requested action is in the action cache and retrieves
     * it; otherwise instantiates a new action instance and puts it in te cache
     * @param aUri the requested uri
     * @return an Action instance
     * @throws ServletException if something goes wrong
     */

    private Action getAction(String aUri) throws ServletException {
        Action action;
        int posActionMarker = aUri.indexOf(INIT_PARAM_URI_ACTION_MARKER);
        if (posActionMarker > -1) {
            Class actionClass = null;
            try {
                String actionClassName = null;
                actionClassName = INIT_PARAM_ACTIONS_PACKAGE + '.' + aUri.substring(posActionMarker + INIT_PARAM_URI_ACTION_MARKER.length());

                action = (Action) actionsCache.get(actionClassName);
                if (action == null) {
                    actionClass = Class.forName(actionClassName);
                    action = (Action) actionClass.newInstance();
                    actionsCache.put(actionClassName, action);
                }
            } catch (InstantiationException e) {
                throw new ServletException("Unable to instantiate action: " + aUri, e);
            } catch (IllegalAccessException e) {
                throw new ServletException("Unable to instantiate action: " + aUri, e);
            } catch (Exception e) {
                throw new ServletException("Action not found: " + aUri, e);
            }

        } else {
            throw new ServletException("Invalid action! - " + aUri);
        }
        return action;
    }

    /**
     * initializes the init parameters if they are not already inialized
     * @throws ServletException if the setting is not specified in web.xml
     */
    private void initializeInitParams() throws ServletException {
        if (INIT_PARAM_ACTIONS_PACKAGE == null) {
            INIT_PARAM_ACTIONS_PACKAGE = getInitParameter("ACTIONS_PACKAGE");
            if (INIT_PARAM_ACTIONS_PACKAGE == null) {
                throw new ServletException("Please specify the ACTIONS_PACKAGE init parameter in web.xml");
            }
        }

        if (INIT_PARAM_URI_ACTION_MARKER == null) {
            INIT_PARAM_URI_ACTION_MARKER = getInitParameter("URI_ACTION_MARKER");
            if (INIT_PARAM_URI_ACTION_MARKER == null) {
                throw new ServletException("Please specify the URI_ACTION_MARKER init parameter in web.xml");
            }
        }
    }


    /**
     * Actions are instantiated and put in this cache on first use.
     */
    private static Hashtable actionsCache = new Hashtable();

    private static String INIT_PARAM_ACTIONS_PACKAGE = null;
    private static String INIT_PARAM_URI_ACTION_MARKER = null;
}
