package inet.warehouse.web.actions;

import inet.warehouse.web.Action;
import inet.warehouse.web.ActionException;
import inet.warehouse.web.Constants;
import inet.warehouse.model.User;
import inet.warehouse.model.AModel;
import inet.warehouse.model.Warehouse;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * Adds a new user
 */
public class AddUser extends Action {
    public static final String PAGE_FAIL = "/newUser.jsp";
    public static final String PAGE_OK = "/newUser.jsp";

    public static final String REQ_USERNAME = "userName";
    public static final String REQ_NAME = "name";
    public static final String REQ_PASSWORD = "password";
    public static final String REQ_PASSWORD_VERIFY = "passwordVerify";

    public static final String VAR_CLOSE = "close";

    public String execute(HttpServletRequest request, HttpServletResponse response) throws ActionException {
        try {
            String username = request.getParameter(REQ_USERNAME);
            String name = request.getParameter(REQ_NAME);
            String password = request.getParameter(REQ_PASSWORD);
            String passwordVerify = request.getParameter(REQ_PASSWORD_VERIFY);

            // check for existing user
            User[] all = (User[]) AModel.findBy(User.class, "userName", username);
            if (all.length > 0) {
                addError(request,  "errors.userExsists" ,new Object[]{username});
                return PAGE_FAIL;
            }

            // password verofication
            if (!password.equals(passwordVerify)) {
                addError(request, "errors.BothPasswordsDoNotMatch");
                return PAGE_FAIL;
            }

            // everithing's ok - create the new user
            User user = new User();
            user.setUserName(username);
            user.setName(name);
            user.setPassword(password);

            Warehouse wh = new Warehouse("SystemDefault","SystemDefault",user);
            wh.save();
            user.save();

            request.setAttribute(VAR_CLOSE,"true");
            return PAGE_OK;
        } catch (Exception e) {
            throw new ActionException(e);
        }
    }
}
