package inet.warehouse.web.tags;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.tagext.BodyTagSupport;
import javax.servlet.jsp.tagext.Tag;
import java.io.IOException;

public class MenuItemTag extends BodyTagSupport {
    /**
     * CSS style for a menu item
     */
    public static final String CSS_CLASS_MENUITEM = "menuItem";

    /**
     * CSS style for a selected menu item
     */
    public static final String CSS_CLASS_MENUITEM_SELECTED = "menuItemSelected";


    public MenuItemTag() {
        super();
        resetValues();
    }

    public int doStartTag() throws JspException {
        checkParent();
        initialize();

        try {
            render();
        } catch (IOException e) {
            throw new JspException(e);
        }

        return EVAL_BODY_BUFFERED;
    }

    public int doAfterBody() throws JspException {
        boolean renderBody;
        if (this.hasSelectedChildren) {
            renderBody = true;
        } else {
            if (this.expandSelectedOnly) {
                if (this.selected) {
                    renderBody = true;
                } else {
                    renderBody = false;
                }
            } else {
                renderBody = true;
            }
        }

        try {
            if (renderBody) {
                JspWriter out = getBodyContent().getEnclosingWriter();
                out.println(getBodyContent().getString());
            }
        } catch (IOException e) {
            throw new JspException(e);
        }
        return SKIP_BODY;
    }

    public int doEndTag() throws JspException {
        resetValues();
        return EVAL_PAGE;
    }

    public void release() {
        resetValues();
    }

    private void resetValues() {
        this.text = "";
        this.url = "";
        this.target = "";
        this.iconUrl = "";
        this.size = "";
        this.id = "";
        this.selected = false;
        this.expandSelectedOnly = false;
        this.hasSelectedChildren = false;
        this.level = 1;
    }

    public String getText() {
        return this.text;
    }

    public void setText(String aText) {
        this.text = aText;
    }

    public String getIconUrl() {
        return this.iconUrl;
    }

    public void setIconUrl(String aIconUrl) {
        this.iconUrl = aIconUrl;
    }

    public String getUrl() {
        return this.url;
    }

    public void setUrl(String aUrl) {
        this.url = aUrl;
    }

    public String getTarget() {
        return target;
    }

    public void setTarget(String aTarget) {
        target = aTarget;
    }

    public String getSize() {
        return this.size;
    }

    public void setSize(String aSize) {
        this.size = aSize;
    }

    public String getId() {
        return this.id;
    }

    public void setId(String id) {
        if ((id == null) || (id.trim().length() == 0)) {
            throw new IllegalArgumentException("The menu id cannot be empty");
        }

        if (getParent() instanceof MenuTag) {
            this.id = ((MenuTag)getParent()).getId() + "." + id;
        } else if (getParent() instanceof MenuItemTag) {
            this.id = ((MenuItemTag)getParent()).getId() + "." + id;
        } else {
            this.id = id;
        }
    }

    public String getSelected() {
        return "" + this.selected;
    }

    public void setSelected(String aSelected) {
        this.selected = Boolean.valueOf(aSelected).booleanValue();
    }

    public boolean isHasSelectedChildren() {
        return this.hasSelectedChildren;
    }

    public void setHasSelectedChildren(boolean aHasSelectedChildren) {
        this.hasSelectedChildren = aHasSelectedChildren;
    }

    private void checkParent() {
        if (!(getParent() instanceof MenuTag) && !(getParent() instanceof MenuItemTag)) {
            errorNesting();
        }
    }

    private static void errorNesting() {
        throw new IllegalArgumentException("The menuitem tag must be nested in a menu tag or another menuitem!");
    }

    /**
     * Renders the tag
     * @throws IOException
     */
    private void render() throws IOException {
        HttpServletRequest request = (HttpServletRequest) pageContext.getRequest();
        JspWriter out = pageContext.getOut();

        StringBuffer strTarget = new StringBuffer();
        if (this.target.length() > 0) {
            strTarget.append("target=\"");
            strTarget.append(this.target);
            strTarget.append("\"");
        }

        StringBuffer strIcon = new StringBuffer();
        if (this.iconUrl.length() > 0) {
            strIcon.append("<img src=\"");
            strIcon.append(fixContextRelativeUrl(request, this.iconUrl));
            strIcon.append("\" border=\"0\" ");
            if (this.size.length() > 0) {
                strIcon.append("width=\"");
                strIcon.append(this.size);
                strIcon.append("\" height=\"");
                strIcon.append(this.size);
                strIcon.append("\"");
            }
            strIcon.append("/>");
        }

        StringBuffer strIndent = new StringBuffer();
        if (this.level > 1) {
            for (int idx=1; idx < this.level; idx++) {
                strIndent.append(INDENT);
            }
        }

        StringBuffer strAnchorStart = new StringBuffer();
        strAnchorStart.append("<a href=\"");
        strAnchorStart.append(fixContextRelativeUrl(request, this.url));
        strAnchorStart.append("\" ");
        strAnchorStart.append(strTarget);
        strAnchorStart.append(" >");
        String strAnchorEnd = "</a>";

        String cssClass = (this.selected) ? CSS_CLASS_MENUITEM_SELECTED : CSS_CLASS_MENUITEM;

        out.print("<tr><td id=\"");
        out.print(this.id);
        out.println("\" valign=\"middle\" align=\"left\" class=\""+ cssClass + "\" nowarp>");

        if (strIcon.length() > 0) {
            out.print(strAnchorStart.toString());
            out.print(strIcon.toString());
            out.println(strAnchorEnd);
        }
        out.print(" ");
        out.print(strIndent);
        out.print(strAnchorStart.toString());
        out.print(this.text);
        out.println(strAnchorEnd);

        out.println("</td></tr>");
    }

    /**
     * Initializes local variables: selected, expandSelectedOnly and level; sets hasSelectedChildred
     * to all its parents
     */
    private void initialize() {
        HttpServletRequest request = (HttpServletRequest) pageContext.getRequest();
        this.selected = request.getRequestURL().indexOf(this.url) >= 0;

        // check for:
        //   - nesting level: count of all parents that are MenuItemTag instances
        //   - expandSelectedOnly: get method of the main MenuTag
        Tag t = this.getParent();
        this.level = 1;
        while (t != null) {
            if (t instanceof MenuItemTag) {
                this.level++;
                if (this.selected) {
                    ((MenuItemTag)t).setHasSelectedChildren(true);
                }
            } else if (t instanceof MenuTag) {
                this.expandSelectedOnly = ((MenuTag)t).expandSelectedOnly();
            }
            t = t.getParent();
        }
    }

    /**
     * If the parameter URL starts with ~ it is replaced with the context path
     * @param request
     * @param aUrl
     * @return context relative url
     */
    public static final String fixContextRelativeUrl(HttpServletRequest request,  String aUrl) {
        if (aUrl == null) {
            return "";
        }

        int pos = aUrl.indexOf('~');
        if (pos == 0) {
            return request.getContextPath() + aUrl.substring(1);
        } else {
            return aUrl;
        }
    }

    // default values - see resetValues()
    private String text;
    private String url;
    private String target;
    private String iconUrl;
    private String size;
    private String id;

    private boolean selected;
    private boolean expandSelectedOnly;
    private boolean hasSelectedChildren;
    private int level;

    private static final String INDENT = "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;";


    public String toString() {
        return "MenuItemTag: " + this.id;
    }
}
